@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 #
 # This file is part of ReText
-# Copyright: 2012-2021 Dmitry Shachnev
+# Copyright: 2012-2022 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,10 +23,10 @@ import weakref
 from markups import MarkdownMarkup, ReStructuredTextMarkup, TextileMarkup
 from ReText import globalSettings, settings, tablemode
 
-from PyQt5.QtCore import pyqtSignal, QFileInfo, QPoint, QRect, QSize, Qt
-from PyQt5.QtGui import QColor, QImage, QKeyEvent, QMouseEvent, QPainter, \
+from PyQt6.QtCore import pyqtSignal, QFileInfo, QPoint, QRect, QSize, Qt
+from PyQt6.QtGui import QAction, QColor, QImage, QKeyEvent, QMouseEvent, QPainter, \
 QPalette, QTextCursor, QTextFormat, QWheelEvent, QGuiApplication
-from PyQt5.QtWidgets import QAction, QApplication, QFileDialog, QLabel, QTextEdit, QWidget
+from PyQt6.QtWidgets import QApplication, QFileDialog, QLabel, QTextEdit, QWidget
 
 try:
 	from ReText.fakevimeditor import ReTextFakeVimHandler
@@ -126,6 +126,7 @@ class ReTextEdit(QTextEdit):
 		Qt.Key.Key_Underscore,
 		Qt.Key.Key_Asterisk,
 		Qt.Key.Key_QuoteDbl,
+		Qt.Key.Key_QuoteLeft,
 		Qt.Key.Key_Apostrophe
 	]
 	surroundKeysOtherClose = {
@@ -161,7 +162,7 @@ class ReTextEdit(QTextEdit):
 			self.setLineWrapMode(QTextEdit.LineWrapMode.WidgetWidth)
 
 	def updateFont(self):
-		self.setFont(globalSettings.editorFont)
+		self.setFont(globalSettings.getEditorFont())
 		metrics = self.fontMetrics()
 		self.marginx = (int(self.document().documentMargin())
 			+ metrics.horizontalAdvance(' ' * globalSettings.rightMargin))
@@ -186,7 +187,7 @@ class ReTextEdit(QTextEdit):
 	def wheelEvent(self, event):
 		modifiers = QGuiApplication.keyboardModifiers()
 		if modifiers == Qt.KeyboardModifier.ControlModifier:
-			font = globalSettings.editorFont
+			font = globalSettings.getEditorFont()
 			size = font.pointSize()
 			scroll = event.angleDelta().y()
 			if scroll > 0:
@@ -231,15 +232,18 @@ class ReTextEdit(QTextEdit):
 		cursor = self.cursorForPosition(event.pos())
 		curchar = self.document().characterAt(cursor.position())
 		isalpha = curchar.isalpha()
-		dictionary = self.tab.highlighter.dictionary
 		word = None
 		if isalpha and not (oldcursor.hasSelection() and oldcursor.selectedText() != cursor.selectedText()):
 			cursor.select(QTextCursor.SelectionType.WordUnderCursor)
 			word = cursor.selectedText()
 
-		if word is not None and dictionary and not dictionary.check(word):
+		correct = True
+		if word is not None and self.tab.highlighter.dictionaries:
+			correct = any(dictionary.check(word) for dictionary in self.tab.highlighter.dictionaries)
+
+		if not correct:
 			self.setTextCursor(cursor)
-			suggestions = dictionary.suggest(word)
+			suggestions = self.tab.highlighter.dictionaries[0].suggest(word)
 			actions = [self.parent.act(sug, trig=self.fixWord(sug)) for sug in suggestions]
 			menu.insertSeparator(menu.actions()[0])
 			for action in actions[::-1]:
@@ -264,11 +268,10 @@ class ReTextEdit(QTextEdit):
 		block = cursor.block()
 		cursor.clearSelection()
 		self.setTextCursor(cursor)
-		dictionary = self.tab.highlighter.dictionary
-		if (dictionary is None) or not newword:
-			return
-		dictionary.add(newword)
-		self.tab.highlighter.rehighlightBlock(block)
+		if self.tab.highlighter.dictionaries and newword:
+			dictionary = self.tab.highlighter.dictionaries[0]
+			dictionary.add(newword)
+			self.tab.highlighter.rehighlightBlock(block)
 
 	def isSurroundKey(self, key):
 		return key in self.surroundKeysSelfClose or key in self.surroundKeysOtherClose
