@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from PyQt5.QtCore import QPoint
+from PyQt6.QtCore import QPoint
 
 class SyncScroll:
 
@@ -61,10 +61,10 @@ class SyncScroll:
     def _handleLoadFinished(self):
         self.frame.setScrollPosition(self.previewPositionBeforeLoad)
         self.contentIsLoading = False
-        self._recalculatePositionMap()
+        self.frame.getPositionMap(self._setPositionMap)
 
     def _handlePreviewResized(self):
-        self._recalculatePositionMap()
+        self.frame.getPositionMap(self._setPositionMap)
         self._updatePreviewScrollPosition()
         if not self.posmap and self.frame.scrollPosition().y() == 0:
             self.frame.setScrollPosition(self.previewPositionBeforeLoad)
@@ -134,9 +134,6 @@ class SyncScroll:
         preview_scroll_offset = preview_pixel_to_scroll_to - distance_to_top_of_viewport
 
         pos = self.frame.scrollPosition()
-        # scrollPosition() returns QPoint for WebKit and QPointF for WebEngine
-        if isinstance(pos, QPoint):
-            preview_scroll_offset = int(preview_scroll_offset)
         pos.setY(preview_scroll_offset)
         self.frame.setScrollPosition(pos)
 
@@ -144,30 +141,3 @@ class SyncScroll:
         self.posmap = posmap
         if posmap:
             self.posmap[0] = 0
-
-    def _recalculatePositionMap(self):
-        if hasattr(self.frame, 'getPositionMap'):
-            # For WebEngine the update has to be asynchronous
-            self.frame.getPositionMap(self._setPositionMap)
-            return
-
-        # Create a list of input line positions mapped to vertical pixel positions in the preview
-        self.posmap = {}
-        elements = self.frame.findAllElements('[data-posmap]')
-
-        if elements:
-            # If there are posmap attributes, then build a posmap
-            # dictionary from them that will be used whenever the
-            # cursor is moved.
-            for el in elements:
-                value = el.attribute('data-posmap', 'invalid')
-                bottom = el.geometry().bottom()
-
-                # Ignore data-posmap entries that do not have integer values
-                try:
-                    self.posmap[int(value)] = bottom
-                except ValueError:
-                    pass
-
-            self.posmap[0] = 0
-
