@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # This file is part of ReText
-# Copyright: 2017-2020 Dmitry Shachnev
+# Copyright: 2017-2022 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,17 +17,32 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ReText import globalSettings
-from ReText.preview import ReTextWebPreview
 from ReText.syncscroll import SyncScroll
-from PyQt5.QtCore import QEvent, Qt
-from PyQt5.QtGui import QDesktopServices, QGuiApplication
-from PyQt5.QtWebEngineWidgets import QWebEnginePage, QWebEngineView, QWebEngineSettings
+from PyQt6.QtCore import QEvent, Qt
+from PyQt6.QtGui import QDesktopServices, QGuiApplication, QTextDocument
+from PyQt6.QtWebEngineCore import (
+    QWebEnginePage,
+    QWebEngineSettings,
+    QWebEngineUrlRequestInfo,
+    QWebEngineUrlRequestInterceptor,
+)
+from PyQt6.QtWebEngineWidgets import QWebEngineView
+
+
+class ReTextWebEngineUrlRequestInterceptor(QWebEngineUrlRequestInterceptor):
+    def interceptRequest(self, info):
+        if (info.resourceType() == QWebEngineUrlRequestInfo.ResourceType.ResourceTypeXhr
+                and info.requestUrl().isLocalFile()):
+            # For security reasons, disable XMLHttpRequests to local files
+            info.block(True)
 
 
 class ReTextWebEnginePage(QWebEnginePage):
     def __init__(self, parent, tab):
         QWebEnginePage.__init__(self, parent)
         self.tab = tab
+        self.interceptor = ReTextWebEngineUrlRequestInterceptor(self)
+        self.setUrlRequestInterceptor(self.interceptor)
 
     def setScrollPosition(self, pos):
         self.runJavaScript("window.scrollTo(%s, %s);" % (pos.x(), pos.y()))
@@ -70,7 +85,7 @@ class ReTextWebEnginePage(QWebEnginePage):
         return False
 
 
-class ReTextWebEnginePreview(ReTextWebPreview, QWebEngineView):
+class ReTextWebEnginePreview(QWebEngineView):
 
     def __init__(self, tab,
                  editorPositionToSourceLineFunc,
@@ -83,23 +98,32 @@ class ReTextWebEnginePreview(ReTextWebPreview, QWebEngineView):
         self.syncscroll = SyncScroll(webPage,
                                      editorPositionToSourceLineFunc,
                                      sourceLineToEditorPositionFunc)
-        ReTextWebPreview.__init__(self, tab.editBox)
+        self.editBox = tab.editBox
 
         settings = self.settings()
-        settings.setAttribute(QWebEngineSettings.LocalContentCanAccessFileUrls,
-                              False)
+        settings.setDefaultTextEncoding('utf-8')
+        settings.setAttribute(QWebEngineSettings.WebAttribute.LocalContentCanAccessRemoteUrls, True)
 
-    def updateFontSettings(self):
+        # Events relevant to sync scrolling
+        self.editBox.cursorPositionChanged.connect(self._handleCursorPositionChanged)
+        self.editBox.verticalScrollBar().valueChanged.connect(self.syncscroll.handleEditorScrolled)
+        self.editBox.resized.connect(self._handleEditorResized)
+
+        # Scroll the preview when the mouse wheel is used to scroll
+        # beyond the beginning/end of the editor
+        self.editBox.scrollLimitReached.connect(self._handleWheelEvent)
+
+    def setFont(self, font):
         settings = self.settings()
-        settings.setFontFamily(QWebEngineSettings.StandardFont,
-                               globalSettings.font.family())
-        settings.setFontSize(QWebEngineSettings.DefaultFontSize,
-                             globalSettings.font.pointSize())
+        settings.setFontFamily(QWebEngineSettings.FontFamily.StandardFont,
+                               font.family())
+        settings.setFontSize(QWebEngineSettings.FontSize.DefaultFontSize,
+                             font.pointSize())
 
     def setHtml(self, html, baseUrl):
         # A hack to prevent WebEngine from stealing the focus
         self.setEnabled(False)
-        QWebEngineView.setHtml(self, html, baseUrl)
+        super().setHtml(html, baseUrl)
         self.setEnabled(True)
 
     def _handleWheelEvent(self, event):
@@ -110,15 +134,46 @@ class ReTextWebEnginePreview(ReTextWebPreview, QWebEngineView):
 
     def event(self, event):
         # Work-around https://bugreports.qt.io/browse/QTBUG-43602
-        if event.type() == QEvent.ChildAdded:
+        if event.type() == QEvent.Type.ChildAdded:
             event.child().installEventFilter(self)
-        elif event.type() == QEvent.ChildRemoved:
+        elif event.type() == QEvent.Type.ChildRemoved:
             event.child().removeEventFilter(self)
         return super().event(event)
 
     def eventFilter(self, object, event):
-        if event.type() == QEvent.Wheel:
-            if QGuiApplication.keyboardModifiers() == Qt.ControlModifier:
+        if event.type() == QEvent.Type.Wheel:
+            if QGuiApplication.keyboardModifiers() == Qt.KeyboardModifier.ControlModifier:
                 self.wheelEvent(event)
                 return True
         return False
+
+    def findText(self, text, flags):
+        options = QWebEnginePage.FindFlag(0)
+        if flags & QTextDocument.FindFlag.FindBackward:
+            options |= QWebEnginePage.FindFlag.FindBackward
+        if flags & QTextDocument.FindFlag.FindCaseSensitively:
+            options |= QWebEnginePage.FindFlag.FindCaseSensitively
+        super().findText(text, options)
+        return True
+
+    def disconnectExternalSignals(self):
+        self.editBox.cursorPositionChanged.disconnect(self._handleCursorPositionChanged)
+        self.editBox.verticalScrollBar().valueChanged.disconnect(self.syncscroll.handleEditorScrolled)
+        self.editBox.resized.disconnect(self._handleEditorResized)
+
+        self.editBox.scrollLimitReached.disconnect(self._handleWheelEvent)
+
+    def _handleCursorPositionChanged(self):
+        editorCursorPosition = self.editBox.verticalScrollBar().value() + \
+                       self.editBox.cursorRect().top()
+        self.syncscroll.handleCursorPositionChanged(editorCursorPosition)
+
+    def _handleEditorResized(self, rect):
+        self.syncscroll.handleEditorResized(rect.height())
+
+    def wheelEvent(self, event):
+        if QGuiApplication.keyboardModifiers() == Qt.KeyboardModifier.ControlModifier:
+            zoomFactor = self.zoomFactor()
+            zoomFactor *= 1.001 ** event.angleDelta().y()
+            self.setZoomFactor(zoomFactor)
+        return super().wheelEvent(event)
